@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -28,6 +28,13 @@
 
 // common Windows parts
 #include "winvideo.h"
+
+#include <dxsdkver.h>
+#if _DXSDK_PRODUCT_MAJOR >= 9
+// new implementation based on Direct2D
+#include "d2dvideo.cpp"
+#else // _DXSDK_PRODUCT_MAJOR >= 9
+
 // and another headers
 #include <cassert>
 #include <stdio.h>
@@ -505,7 +512,7 @@ DWORD WINAPI thread_vsync(LPVOID lpParameter)
 
 ///////////////////////////////////////////// public methods of video class ///////////////////////
 
-inline void mask2bits(unsigned int mask, color_t &save, char &shift)
+inline void mask2bits(unsigned int mask, color_t &save, depth_t &shift)
 {
     save  = mask; if(!mask) { shift = 8; return; }
     shift = 0; while(!(mask&1)) ++shift, mask >>= 1;
@@ -535,8 +542,8 @@ bool video::init_window(int sizex, int sizey)
     mask2bits(PixelFormat.dwGBitMask, green_mask, green_shift);
     mask2bits(PixelFormat.dwBBitMask, blue_mask, blue_shift);
     if(PixelFormat.dwFlags == DDPF_RGB)
-         depth = char(PixelFormat.dwRGBBitCount);
-    else depth = -char(PixelFormat.dwFourCC);
+         depth = depth_t(PixelFormat.dwRGBBitCount);
+    else depth = -depth_t(PixelFormat.dwFourCC);
     for(int i = 0, e = sizex * sizey * PixelFormat.dwRGBBitCount / 32, c = get_color(0, 0, 0); i < e; i++)
         g_pImg[i] = c; // clear surface
     ShowWindow(g_hAppWnd, SW_SHOW);
@@ -575,6 +582,8 @@ drawing_area::drawing_area(int x, int y, int sizex, int sizey)
     index = base_index; // current index
 }
 
-drawing_area::~drawing_area()
+void drawing_area::update()
 {
 }
+
+#endif //_DXSDK_PRODUCT_MAJOR >= 9
