@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -48,8 +48,10 @@ typedef signed char depth_t;
 //! Class for getting access to drawing memory
 class drawing_memory
 {
+#ifdef __TBB_MIC_OFFLOAD
     // The address is kept as uintptr_t since
     // the compiler could not offload a pointer
+#endif
     uintptr_t   my_address;
 public:
     depth_t     pixel_depth;
@@ -105,6 +107,8 @@ public:
     //! Get drawing memory descriptor
     inline drawing_memory get_drawing_memory() const;
 
+    //! code of the ESCape key
+    static const int esc_key = 27;
     //! Mouse events handler.
     virtual void on_mouse(int x, int y, int key) { }
     //! Mouse events handler.
@@ -172,7 +176,7 @@ inline color_t video::get_color(colorcomp_t red, colorcomp_t green, colorcomp_t
         register depth_t bs = blue_shift, rs = red_shift;
         if(blue_shift < 0) blue >>= -bs, bs = 0;
         else /*red_shift < 0*/ red >>= -rs, rs = 0;
-        return (red<<rs)&red_mask | (green<<green_shift)&green_mask | (blue<<bs)&blue_mask;
+        return ((red<<rs)&red_mask) | ((green<<green_shift)&green_mask) | ((blue<<bs)&blue_mask);
     } else { // UYVY colorspace
         register unsigned y, u, v;
         y = red * 77 + green * 150 + blue * 29; // sum(77+150+29=256) * max(=255):  limit->2^16
@@ -213,7 +217,9 @@ inline void drawing_area::put_pixel(color_t color)
 
 inline drawing_area::~drawing_area()
 {
+#if ! __TBB_DEFINE_MIC
     update();
+#endif
 }
 
 #if defined(_WINDOWS) && (defined(VIDEO_WINMAIN) || defined(VIDEO_WINMAIN_ARGS) )
