@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -29,6 +29,9 @@
 /////// Common internal implementation of Windows-specific stuff //////////////
 ///////                  Must be the first included header       //////////////
 
+#ifndef __WINVIDEO_H__
+#define __WINVIDEO_H__
+
 #ifndef _CRT_SECURE_NO_DEPRECATE
 #define _CRT_SECURE_NO_DEPRECATE
 #endif
@@ -217,11 +220,9 @@ static bool loop_once(video *v)
     }
     // event processing, including painting
     MSG msg;
-    if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
-    {
+    if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)){
         if( msg.message == WM_QUIT ) { v->running = false; return false; }
-        if( !hAccelTable || !TranslateAccelerator(msg.hwnd, hAccelTable, &msg) )
-        {
+        if( !hAccelTable || !TranslateAccelerator(msg.hwnd, hAccelTable, &msg) ){
             TranslateMessage(&msg);
             DispatchMessage(&msg);
         }
@@ -256,8 +257,10 @@ void video::main_loop()
             if(r == WAIT_OBJECT_0) break; // thread terminated
         }
         running = false;
-        if(WaitForSingleObject(g_handles[0], 300) == WAIT_TIMEOUT)
-            TerminateThread(g_handles[0], 0);
+        if(WaitForSingleObject(g_handles[0], 3000) == WAIT_TIMEOUT){
+            // there was not enough time for graceful shutdown, killing the example with code 1.
+            exit(1);
+        }
         if(g_handles[0]) CloseHandle(g_handles[0]);
         if(g_handles[1]) CloseHandle(g_handles[1]);
         g_handles[0] = g_handles[1] = 0;
@@ -284,3 +287,5 @@ void video::show_title()
     if(g_hAppWnd)
         SetWindowTextA(g_hAppWnd, title);
 }
+
+#endif //__WINVIDEO_H__
