@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -32,17 +32,11 @@
 #include "tbb/tick_count.h"
 #include "../../common/utility/utility.h"
 
-#if __TBB_MIC_OFFLOAD
-#pragma offload_attribute (push,target(mic))
-#endif // __TBB_MIC_OFFLOAD
 #include "seismic_video.h"
 #include "universe.h"
 #include "tbb/task_scheduler_init.h"
 
 Universe u;
-#if __TBB_MIC_OFFLOAD
-#pragma offload_attribute (pop)
-#endif // __TBB_MIC_OFFLOAD
 
 struct RunOptions {
     //! It is used for console mode for test with different number of threads and also has
@@ -60,9 +54,9 @@ struct RunOptions {
 
 int do_get_default_num_threads() {
     int threads;
-    #if __TBB_MIC_OFFLOAD
+#if __TBB_MIC_OFFLOAD
     #pragma offload target(mic) out(threads)
-    #endif // __TBB_MIC_OFFLOAD
+#endif // __TBB_MIC_OFFLOAD
     threads = tbb::task_scheduler_init::default_num_threads();
     return threads;
 }
@@ -113,7 +107,7 @@ int main(int argc, char *argv[])
                 tbb::tick_count xwayParallelismStartTime = tbb::tick_count::now();
                 u.InitializeUniverse(video);
                 int numberOfFrames = options.numberOfFrames;
-                #if __TBB_MIC_OFFLOAD
+#if __TBB_MIC_OFFLOAD
                 drawing_memory dmem = video.get_drawing_memory();
                 char *pMem = dmem.get_address();
                 size_t memSize = dmem.get_size();
@@ -124,21 +118,21 @@ int main(int argc, char *argv[])
                     // since the address spaces on host and on target are different
                     dmem.set_address(pMem);
                     u.SetDrawingMemory(dmem);
-                #endif // __TBB_MIC_OFFLOAD
-                if (p==0){
-                    //run a serial version
-                    for( int i=0; i<numberOfFrames; ++i ){
-                        u.SerialUpdateUniverse();
-                    }
-                }else{
-                    tbb::task_scheduler_init init(p);
-                    for( int i=0; i<numberOfFrames; ++i ){
-                        u.ParallelUpdateUniverse();
+#endif // __TBB_MIC_OFFLOAD
+                    if (p==0){
+                        //run a serial version
+                        for( int i=0; i<numberOfFrames; ++i ){
+                            u.SerialUpdateUniverse();
+                        }
+                    }else{
+                        tbb::task_scheduler_init init(p);
+                        for( int i=0; i<numberOfFrames; ++i ){
+                            u.ParallelUpdateUniverse();
+                        }
                     }
+#if __TBB_MIC_OFFLOAD
                 }
-                #if __TBB_MIC_OFFLOAD
-                }
-                #endif // __TBB_MIC_OFFLOAD
+#endif // __TBB_MIC_OFFLOAD
 
                 if (!options.silent){
                     double fps =  options.numberOfFrames/((tbb::tick_count::now()-xwayParallelismStartTime).seconds());
