@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,19 +26,34 @@
     the GNU General Public License.
 */
 
+#if __TBB_MIC_OFFLOAD
+#pragma offload_attribute (push,target(mic))
+#endif // __TBB_MIC_OFFLOAD
+
+// This header should come before any other one.
+// For details, see Known Issues in the Release Notes.
+#include "tbb/tbb_stddef.h"
 
 #include <cstdio>
 #include <cstdlib>
 #include <string>
+
 #include "tbb/atomic.h"
 #include "tbb/tick_count.h"
 #include "tbb/task_scheduler_init.h"
 #include "tbb/task_group.h"
+
 #include "../../common/utility/utility.h"
 
 #pragma warning(disable: 4996)
 
-#define __TBB_LAMBDAS_PRESENT  ( _MSC_VER >= 1600 && !__INTEL_COMPILER || __INTEL_COMPILER > 1100 && _TBB_CPP0X )
+#if __INTEL_COMPILER
+#define __TBB_LAMBDAS_PRESENT ( _TBB_CPP0X && __INTEL_COMPILER > 1100 )
+#elif __GNUC__
+#define __TBB_LAMBDAS_PRESENT ( _TBB_CPP0X && __TBB_GCC_VERSION >= 40500 )
+#elif _MSC_VER
+#define __TBB_LAMBDAS_PRESENT ( _MSC_VER>=1600 )
+#endif
 
 const unsigned BOARD_SIZE=81;
 const unsigned BOARD_DIM=9;
@@ -264,11 +279,21 @@ unsigned solve(int p) {
     return nSols;
 }
 
+#if __TBB_MIC_OFFLOAD
+#pragma offload_attribute (pop)
+#endif // __TBB_MIC_OFFLOAD
+
+int do_get_default_num_threads() {
+    int threads;
+    #if __TBB_MIC_OFFLOAD
+    #pragma offload target(mic) out(threads)
+    #endif // __TBB_MIC_OFFLOAD
+    threads = tbb::task_scheduler_init::default_num_threads();
+    return threads;
+}
 
 int get_default_num_threads() {
-    static int threads = 0;
-    if ( threads == 0 )
-        threads = tbb::task_scheduler_init::default_num_threads();
+    static int threads = do_get_default_num_threads();
     return threads;
 }
 
@@ -282,8 +307,8 @@ int main(int argc, char *argv[]) {
 
         utility::parse_cli_arguments(argc,argv,
             utility::cli_argument_pack()
-            //"-h" option for for displaying help is present implicitly
-            .positional_arg(threads,"n-of-threads","number of threads to use; a range of the form low[:high], where low and optional high are non-negative integers or 'auto' for the TBB default.")
+            //"-h" option for displaying help is present implicitly
+            .positional_arg(threads,"n-of-threads",utility::thread_number_range_desc)
             .positional_arg(filename,"filename","input filename")
 
             .arg(verbose,"verbose","prints the first solution")
@@ -296,9 +321,16 @@ int main(int argc, char *argv[]) {
         if ( !filename.empty() )
             read_board( filename.c_str() );
         // otherwise (if file name not specified), the default statically initialized board will be used.
-        for(int p = threads.first; p <= threads.last; ++p ) {
+        for(int p = threads.first; p <= threads.last; p = threads.step(p) ) {
             unsigned number;
+            #if __TBB_MIC_OFFLOAD
+            #pragma offload target(mic) in(init_values, p, verbose, find_one) out(number, solve_time)
+            {
+            #endif // __TBB_MIC_OFFLOAD
             number = solve(p);
+            #if __TBB_MIC_OFFLOAD
+            }
+            #endif // __TBB_MIC_OFFLOAD
 
             if ( !silent ) {
                 if ( find_one ) {
