@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -47,12 +47,26 @@
     #define __TBB_WORDSIZE 4
 #endif
 
+// Traditionally Power Architecture is big-endian.
+// Little-endian could be just an address manipulation (compatibility with TBB not verified),
+// or normal little-endian (on more recent systems). Embedded PowerPC systems may support
+// page-specific endianness, but then one endianness must be hidden from TBB so that it still sees only one.
+#if __BIG_ENDIAN__ || (defined(__BYTE_ORDER__) && __BYTE_ORDER__==__ORDER_BIG_ENDIAN__)
+    #define __TBB_ENDIANNESS __TBB_ENDIAN_BIG
+#elif __LITTLE_ENDIAN__ || (defined(__BYTE_ORDER__) && __BYTE_ORDER__==__ORDER_LITTLE_ENDIAN__)
+    #define __TBB_ENDIANNESS __TBB_ENDIAN_LITTLE
+#elif defined(__BYTE_ORDER__)
+    #define __TBB_ENDIANNESS __TBB_ENDIAN_UNSUPPORTED
+#else
+    #define __TBB_ENDIANNESS __TBB_ENDIAN_DETECT
+#endif
+
 // On Power Architecture, (lock-free) 64-bit atomics require 64-bit hardware:
 #if __TBB_WORDSIZE==8
     // Do not change the following definition, because TBB itself will use 64-bit atomics in 64-bit builds.
     #define __TBB_64BIT_ATOMICS 1
 #elif __bgp__
-    // Do not change the following definition on known 32-bit hardware.
+    // Do not change the following definition, because this is known 32-bit hardware.
     #define __TBB_64BIT_ATOMICS 0
 #else
     // To enable 64-bit atomics in 32-bit builds, set the value below to 1 instead of 0.
@@ -148,6 +162,7 @@ inline int64_t __TBB_machine_cmpswp8 (volatile void *ptr, int64_t value, int64_t
                          );
     return result;
 }
+
 #endif /* __TBB_WORDSIZE==4 && __TBB_64BIT_ATOMICS */
 
 #define __TBB_MACHINE_DEFINE_LOAD_STORE(S,ldx,stx,cmpx)                                                       \
@@ -275,14 +290,16 @@ namespace internal {
 
 #undef __TBB_MACHINE_DEFINE_LOAD_STORE
 
-#define __TBB_USE_GENERIC_PART_WORD_CAS 1
-#define __TBB_USE_GENERIC_FETCH_ADD     1
-#define __TBB_USE_GENERIC_FETCH_STORE   1
+#define __TBB_USE_GENERIC_PART_WORD_CAS                     1
+#define __TBB_USE_GENERIC_FETCH_ADD                         1
+#define __TBB_USE_GENERIC_FETCH_STORE                       1
+#define __TBB_USE_GENERIC_SEQUENTIAL_CONSISTENCY_LOAD_STORE 1
 
 #define __TBB_control_consistency_helper() __asm__ __volatile__("isync": : :"memory")
 #define __TBB_full_memory_fence()          __asm__ __volatile__( "sync": : :"memory")
 
 static inline intptr_t __TBB_machine_lg( uintptr_t x ) {
+    __TBB_ASSERT(x, "__TBB_Log2(0) undefined");
     // cntlzd/cntlzw starts counting at 2^63/2^31 (ignoring any higher-order bits), and does not affect cr0
 #if __TBB_WORDSIZE==8
     __asm__ __volatile__ ("cntlzd %0,%0" : "+r"(x));
