@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -53,7 +53,7 @@ public:
     internal_construct();
 #else
   #if _WIN32||_WIN64
-        InitializeCriticalSection(&impl);
+        InitializeCriticalSectionEx(&impl, 4000, 0);
   #else
         int error_code = pthread_mutex_init(&impl,NULL);
         if( error_code )
