@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -44,7 +44,7 @@
 #include "atomic.h"
 
 #if defined(_MSC_VER) && !defined(__INTEL_COMPILER)
-    // Workaround for overzealous compiler warnings 
+    // Workaround for overzealous compiler warnings
     #pragma warning (push)
     #pragma warning (disable: 4244)
 #endif
@@ -60,7 +60,7 @@ namespace interface6 {
     }
 }
 
-namespace internal {
+namespace internal { //< @cond INTERNAL
 size_t __TBB_EXPORTED_FUNC get_initial_auto_partitioner_divisor();
 
 //! Defines entry point for affinity partitioner into tbb run-time library.
@@ -95,8 +95,7 @@ public:
 
 template<typename Range, typename Body, typename Partitioner> class start_scan;
 
-} // namespace internal
-//! @endcond
+} //< namespace internal @endcond
 
 namespace serial {
 namespace interface6 {
@@ -114,19 +113,20 @@ template<typename Range, typename Body, typename Partitioner> class start_reduce
 //! Join task node that contains shared flag for stealing feedback
 class flag_task: public task {
 public:
-    tbb::atomic<bool> child_stolen;
-    flag_task() { child_stolen = false; }
+    tbb::atomic<bool> my_child_stolen;
+    flag_task() { my_child_stolen = false; }
     task* execute() { return NULL; }
-};
-
-//! Task to signal the demand without carrying the work
-class signal_task: public task {
-public:
-    task* execute() {
-        if( is_stolen_task() ) {
-            static_cast<flag_task*>(parent())->child_stolen = true;
-        }
-        return NULL;
+    static void mark_task_stolen(task &t) {
+        tbb::atomic<bool> &flag = static_cast<flag_task*>(t.parent())->my_child_stolen;
+#if TBB_USE_THREADING_TOOLS
+        // Threading tools respect lock prefix but report false-positive data-race via plain store
+        flag.fetch_and_store<release>(true);
+#else
+        flag = true;
+#endif //TBB_USE_THREADING_TOOLS
+    }
+    static bool is_peer_stolen(task &t) {
+        return static_cast<flag_task*>(t.parent())->my_child_stolen;
     }
 };
 
@@ -204,58 +204,34 @@ struct partition_type_base {
     void note_affinity( task::affinity_id ) {}
     bool check_being_stolen(task &) { return false; } // part of old should_execute_range()
     bool check_for_demand(task &) { return false; }
-    bool divisions_left() { return true; } // part of old should_execute_range()
-    bool should_create_trap() { return false; }
+    bool is_divisible() { return true; } // part of old should_execute_range()
     depth_t max_depth() { return 0; }
     void align_depth(depth_t) { }
     // common function blocks
-    Partition& derived() { return *static_cast<Partition*>(this); }
-    template<typename StartType>
-    flag_task* split_work(StartType &start) {
-        flag_task* parent_ptr = start.create_continuation(); // the type here is to express expectation
-        start.set_parent(parent_ptr);
-        parent_ptr->set_ref_count(2);
-        StartType& right_work = *new( parent_ptr->allocate_child() ) StartType(start, split());
-        start.spawn(right_work);
-        return parent_ptr;
-    }
+    Partition& self() { return *static_cast<Partition*>(this); } // CRTP helper
     template<typename StartType, typename Range>
     void execute(StartType &start, Range &range) {
-        // The algorithm in a few words ([]-denotes calls to decision mathods of partitioner):
+        // The algorithm in a few words ([]-denotes calls to decision methods of partitioner):
         // [If this task is stolen, adjust depth and divisions if necessary, set flag].
         // If range is divisible {
         //    Spread the work while [initial divisions left];
         //    Create trap task [if necessary];
         // }
         // If not divisible or [max depth is reached], execute, else do the range pool part
-        task* parent_ptr = start.parent();
-        if( range.is_divisible() ) {
-            if( derived().divisions_left() )
-                do parent_ptr = split_work(start); // split until divisions_left()
-                while( range.is_divisible() && derived().divisions_left() );
-            if( derived().should_create_trap() ) { // only for range pool
-                if( parent_ptr->ref_count() > 1 ) { // create new parent if necessary
-                    parent_ptr = start.create_continuation();
-                    start.set_parent(parent_ptr);
-                } else __TBB_ASSERT(parent_ptr->ref_count() == 1, NULL);
-                parent_ptr->set_ref_count(2); // safe because parent has only one reference
-                signal_task& right_signal = *new( parent_ptr->allocate_child() ) signal_task();
-                start.spawn(right_signal); // pure signal is to avoid deep recursion in the end
-            }
+        if ( range.is_divisible() ) {
+            if ( self().is_divisible() )
+                do start.offer_work( split() ); // split until is divisible
+                while ( range.is_divisible() && self().is_divisible() );
         }
-        if( !range.is_divisible() || !derived().max_depth() )
+        if( !range.is_divisible() || !self().max_depth() )
             start.run_body( range ); // simple partitioner goes always here
         else { // do range pool
             internal::range_vector<Range, Partition::range_pool_size> range_pool(range);
             do {
-                range_pool.split_to_fill(derived().max_depth()); // fill range pool
-                if( derived().check_for_demand( start ) ) {
+                range_pool.split_to_fill(self().max_depth()); // fill range pool
+                if( self().check_for_demand( start ) ) {
                     if( range_pool.size() > 1 ) {
-                        parent_ptr = start.create_continuation();
-                        start.set_parent(parent_ptr);
-                        parent_ptr->set_ref_count(2);
-                        StartType& right_work = *new( parent_ptr->allocate_child() ) StartType(start, range_pool.front(), range_pool.front_depth());
-                        start.spawn(right_work);
+                        start.offer_work( range_pool.front(), range_pool.front_depth() );
                         range_pool.pop_front();
                         continue;
                     }
@@ -263,7 +239,7 @@ struct partition_type_base {
                         continue; // note: check_for_demand() should guarantee increasing max_depth() next time
                 }
                 start.run_body( range_pool.back() );
-                range_pool.pop_back();  
+                range_pool.pop_back();
             } while( !range_pool.empty() && !start.is_cancelled() );
         }
     }
@@ -290,37 +266,34 @@ struct auto_partition_type_base : partition_type_base<Partition> {
 #endif
     }
     bool check_being_stolen( task &t) { // part of old should_execute_range()
-        if( !my_divisor ) {
-            my_divisor = 1; // todo: replace by on-stack flag (partition_state's member)?
+        if( !my_divisor ) { // if not from the top P tasks of binary tree
+            my_divisor = 1; // TODO: replace by on-stack flag (partition_state's member)?
             if( t.is_stolen_task() ) {
 #if TBB_USE_EXCEPTIONS
                 // RTTI is available, check whether the cast is valid
                 __TBB_ASSERT(dynamic_cast<flag_task*>(t.parent()), 0);
-                // correctess of the cast rely on avoiding the root task for which:
+                // correctness of the cast relies on avoiding the root task for which:
                 // - initial value of my_divisor != 0 (protected by separate assertion)
-                // - is_stolen_task() always return false for the root task.
+                // - is_stolen_task() always returns false for the root task.
 #endif
-                static_cast<flag_task*>(t.parent())->child_stolen = true;
+                flag_task::mark_task_stolen(t);
                 my_max_depth++;
                 return true;
             }
         }
         return false;
     }
-    bool divisions_left() { // part of old should_execute_range()
+    bool is_divisible() { // part of old should_execute_range()
         if( my_divisor > 1 ) return true;
         if( my_divisor && my_max_depth > 1 ) { // can split the task and once more internally. TODO: on-stack flag instead
             // keep same fragmentation while splitting for the local task pool
             my_max_depth--;
-            my_divisor = 0;
+            my_divisor = 0; // decrease max_depth once per task
             return true;
         } else return false;
     }
-    bool should_create_trap() {
-        return my_divisor > 0;
-    }
     bool check_for_demand(task &t) {
-        if( static_cast<flag_task*>(t.parent())->child_stolen ) {
+        if( flag_task::is_peer_stolen(t) ) {
             my_max_depth++;
             return true;
         } else return false;
@@ -347,15 +320,15 @@ class affinity_partition_type : public auto_partition_type_base<affinity_partiti
     }
 public:
     affinity_partition_type( tbb::internal::affinity_partitioner_base_v3& ap ) {
-        __TBB_ASSERT( (factor&(factor-1))==0, "factor must be power of two" ); 
+        __TBB_ASSERT( (factor&(factor-1))==0, "factor must be power of two" );
         ap.resize(factor);
         my_array = ap.my_array;
         map_begin = 0;
         map_end = unsigned(ap.my_size);
         set_mid();
         my_delay = true;
-        my_divisor /= __TBB_INITIAL_CHUNKS; // let excatly P tasks to be distributed across workers
-        my_max_depth = factor_power+1; // the first factor_power ranges will be spawned, and >=1 ranges should left
+        my_divisor /= __TBB_INITIAL_CHUNKS; // let exactly P tasks to be distributed across workers
+        my_max_depth = factor_power+1; // the first factor_power ranges will be spawned, and >=1 ranges should be left
         __TBB_ASSERT( my_max_depth < __TBB_RANGE_POOL_CAPACITY, 0 );
     }
     affinity_partition_type(affinity_partition_type& p, split)
@@ -371,7 +344,7 @@ public:
             t.set_affinity( my_array[map_begin] );
     }
     void note_affinity( task::affinity_id id ) {
-        if( map_begin<map_end ) 
+        if( map_begin<map_end )
             my_array[map_begin] = id;
     }
     bool check_for_demand( task &t ) {
@@ -380,19 +353,16 @@ public:
                 __TBB_ASSERT(my_max_depth>__TBB_Log2(map_end-map_mid), 0);
                 return true;// do not do my_max_depth++ here, but be sure my_max_depth is big enough
             }
-            if( static_cast<flag_task*>(t.parent())->child_stolen ) {
+            if( flag_task::is_peer_stolen(t) ) {
                 my_max_depth++;
                 return true;
             }
         } else my_delay = false;
         return false;
     }
-    bool divisions_left() { // part of old should_execute_range()
+    bool is_divisible() {
         return my_divisor > 1;
     }
-    bool should_create_trap() {
-        return true; // TODO: rethink for the stage after memorizing level
-    }
     static const unsigned range_pool_size = __TBB_RANGE_POOL_CAPACITY;
 };
 
@@ -412,7 +382,7 @@ public:
     template<typename StartType, typename Range>
     void execute(StartType &start, Range &range) {
         while( range.is_divisible() )
-            split_work( start );
+            start.offer_work( split() );
         start.run_body( range );
     }
     //static const unsigned range_pool_size = 1; - not necessary because execute() is overridden
@@ -441,8 +411,8 @@ public:
 //! @endcond
 } // namespace interfaceX
 
-//! A simple partitioner 
-/** Divides the range until the range is not divisible. 
+//! A simple partitioner
+/** Divides the range until the range is not divisible.
     @ingroup algorithms */
 class simple_partitioner {
 public:
@@ -463,7 +433,7 @@ private:
     typedef interface6::internal::simple_partition_type task_partition_type;
 };
 
-//! An auto partitioner 
+//! An auto partitioner
 /** The range is initial divided into several large chunks.
     Chunks are further subdivided into smaller pieces if demand detected and they are divisible.
     @ingroup algorithms */
