@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -36,11 +36,12 @@
 static tbb::atomic<long> FooConstructed;
 static tbb::atomic<long> FooDestroyed;
 
+enum state_t{
+    LIVE=0x1234,
+    DEAD=0xDEAD
+};
+
 class Foo {
-    enum state_t{
-        LIVE=0x1234,
-        DEAD=0xDEAD
-    };
     state_t state;
 public:
     int thread_id;
@@ -58,8 +59,8 @@ public:
         ASSERT( state==LIVE, NULL );
         ++FooDestroyed;
         state=DEAD;
-        thread_id=0xDEAD;
-        serial=0xDEAD;
+        thread_id=DEAD;
+        serial=DEAD;
     }
     void operator=( Foo& item ) {
         ASSERT( item.state==LIVE, NULL );
@@ -83,7 +84,7 @@ static tbb::atomic<long> PopKind[3];
 
 const int M = 10000;
 
-struct Body {
+struct Body: NoAssign {
     tbb::concurrent_queue<Foo>* queue;
     const int nthread;
     Body( int nthread_ ) : nthread(nthread_) {}
@@ -96,8 +97,8 @@ struct Body {
         long sum = 0;
         for( long j=0; j<M; ++j ) {
             Foo f;
-            f.thread_id = 0xDEAD;
-            f.serial = 0xDEAD;
+            f.thread_id = DEAD;
+            f.serial = DEAD;
             bool prepopped = false;
             if( j&1 ) {
                 prepopped = queue->pop_if_present(f);
@@ -245,20 +246,20 @@ void TestIterator() {
     tbb::concurrent_queue<Foo> queue;
     tbb::concurrent_queue<Foo>& const_queue = queue;
     for( int j=0; j<500; ++j ) {
-        TestIteratorAux( queue.begin(), queue.end(), j );
+        TestIteratorAux(       queue.begin(),       queue.end(), j );
         TestIteratorAux( const_queue.begin(), const_queue.end(), j );
-        TestIteratorAux( const_queue.begin(), queue.end(), j );
-        TestIteratorAux( queue.begin(), const_queue.end(), j );
+        TestIteratorAux( const_queue.begin(),       queue.end(), j );
+        TestIteratorAux(       queue.begin(), const_queue.end(), j );
         Foo f;
         f.serial = j+1;
         queue.push(f);
     }
     TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>( const_queue.begin() );
-    TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>( queue.begin() );
-    TestIteratorAssignment<tbb::concurrent_queue<Foo>::iterator>( queue.begin() );
+    TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>(       queue.begin() );
+    TestIteratorAssignment<tbb::concurrent_queue<Foo>::      iterator>(       queue.begin() );
 }
 
-void TestConcurrenetQueueType() {
+void TestConcurrentQueueType() {
     AssertSameType( tbb::concurrent_queue<Foo>::value_type(), Foo() );
     Foo f;
     const Foo g;
@@ -301,7 +302,7 @@ void TestFullQueue() {
 }
 
 template<typename T>
-struct TestNegativeQueueBody {
+struct TestNegativeQueueBody: NoAssign {
     tbb::concurrent_queue<T>& queue;
     const int nthread;
     TestNegativeQueueBody( tbb::concurrent_queue<T>& q, int n ) : queue(q), nthread(n) {}
@@ -338,7 +339,7 @@ int TestMain () {
     TestEmptyQueue<char>();
     TestEmptyQueue<Foo>();
     TestFullQueue();
-    TestConcurrenetQueueType();
+    TestConcurrentQueueType();
     TestIterator();
 
     // Test concurrent operations
