@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -283,7 +283,7 @@ namespace internal {
         public:
             Initializer () {
                 SYSTEM_INFO si;
-                GetSystemInfo(&si);
+                GetNativeSystemInfo(&si);
                 ASSERT( si.dwNumberOfProcessors <= MaxAffinitySetSize, "Too many CPUs" );
                 AffinitySetSize = min (si.dwNumberOfProcessors, MaxAffinitySetSize);
                 cpu_set_t systemMask = 0;
@@ -353,10 +353,12 @@ namespace internal {
 #endif /* __linux__ */
 
     bool PinTheThread ( int cpu_idx, tbb::atomic<int>& nThreads ) {
+    #if _MSC_VER || __linux__
         cpu_set_t orig_mask, target_mask;
         CPU_ZERO( &target_mask );
         CPU_SET( cpu_idx, &target_mask );
         ASSERT ( CPU_ISSET(cpu_idx, &target_mask), "CPU_SET failed" );
+    #endif
     #if _MSC_VER
         orig_mask = SetThreadAffinityMask( GetCurrentThread(), target_mask );
         if ( !orig_mask )
@@ -759,11 +761,14 @@ inline bool __TBB_bool( bool b ) { return b; }
 
     void Report ( char const* fmt, ... ) {
         va_list args;
-        va_start( args, fmt );
-        if ( ResFile )
+        if ( ResFile ) {
+            va_start( args, fmt );
             vfprintf( ResFile, fmt, args );
+            va_end( args );
+        }
         va_start( args, fmt );
         vprintf( fmt, args );
+        va_end( args );
     }
 
     void PrintResults () {
