@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -44,22 +44,26 @@
 #include "task_stream.h"
 #include "../rml/include/rml_tbb.h"
 #include "mailbox.h"
+#include "observer_proxy.h"
+#if __TBB_TASK_ARENA
+#include "concurrent_monitor.h"
+#endif
 
 namespace tbb {
 
+namespace interface6 {
+class task_arena;
+}
 class task_group_context;
 class allocate_root_with_context_proxy;
 
 namespace internal {
 
+class task_scheduler_observer_v3;
 class governor;
 class arena;
 template<typename SchedulerTraits> class custom_scheduler;
 
-//------------------------------------------------------------------------
-// arena
-//------------------------------------------------------------------------
-
 class market;
 
 //! arena data except the array of slots
@@ -84,10 +88,13 @@ struct arena_base : intrusive_list_node {
     //! Number of workers that have been marked out by the resource manager to service the arena
     unsigned my_num_workers_allotted;
 
-    //! Number of threads in the arena at the moment
-    /** Consists of the workers servicing the arena and one master until it starts 
-        arena shutdown and detaches from it. Plays the role of the arena's ref count. **/
-    atomic<unsigned> my_num_threads_active;
+    //! References of the arena
+    /** Counts workers and master references separately. Bit 0 indicates reference from implicit
+        master or explicit task_arena; the next bits contain number of workers servicing the arena.*/
+    atomic<unsigned> my_references;
+
+    //! ABA prevention marker
+    uintptr_t my_aba_epoch;
 
     //! FPU control settings of arena's master thread captured at the moment of arena instantiation.
     __TBB_cpu_ctl_env_t my_cpu_ctl_env;
@@ -107,7 +114,7 @@ struct arena_base : intrusive_list_node {
     //! Default task group context.
     /** Used by root tasks allocated directly by the master thread (not from inside
         a TBB task) without explicit context specification. **/
-    task_group_context* my_master_default_ctx;
+    task_group_context* my_default_ctx;
 #endif /* __TBB_TASK_GROUP_CONTEXT */
 
 #if __TBB_TASK_PRIORITY
@@ -150,9 +157,19 @@ struct arena_base : intrusive_list_node {
     task_stream my_task_stream;
 #endif /* !__TBB_TASK_PRIORITY */
 
+#if __TBB_SCHEDULER_OBSERVER
+    //! List of local observers attached to this arena.
+    observer_list my_observers;
+#endif /* __TBB_SCHEDULER_OBSERVER */
+
     //! Indicates if there is an oversubscribing worker created to service enqueued tasks.
     bool my_mandatory_concurrency;
 
+#if __TBB_TASK_ARENA
+    //! exit notifications after arena slot is released
+    concurrent_monitor my_exit_monitors;
+#endif
+
 #if TBB_USE_ASSERT
     //! Used to trap accesses to the object after its destruction.
     uintptr_t my_guard;
@@ -170,11 +187,17 @@ private:
     friend class generic_scheduler;
     template<typename SchedulerTraits> friend class custom_scheduler;
     friend class governor;
-
+    friend class task_scheduler_observer_v3;
     friend class market;
     friend class tbb::task_group_context;
     friend class allocate_root_with_context_proxy;
     friend class intrusive_list<arena>;
+#if __TBB_TASK_ARENA
+    friend class tbb::interface6::task_arena; // included through in scheduler_common.h
+    friend class interface6::delegated_task;
+    friend class interface6::wait_task;
+    friend struct interface6::wait_body;
+#endif //__TBB_TASK_ARENA
 
     typedef padded<arena_base> base_type;
 
@@ -226,7 +249,7 @@ private:
 
     //! The number of workers active in the arena.
     unsigned num_workers_active( ) {
-        return my_num_threads_active - (my_slots[0].my_scheduler? 1 : 0);
+        return my_references >> 1;
     }
 
     //! If necessary, raise a flag that there is new job in arena.
@@ -236,14 +259,15 @@ private:
     /** Return true if no job or if arena is being cleaned up. */
     bool is_out_of_work();
 
-    //! Initiates arena shutdown.
-    void close_arena ();
+    //! enqueue a task into starvation-resistance queue
+    void enqueue_task( task&, intptr_t, unsigned & );
 
     //! Registers the worker with the arena and enters TBB scheduler dispatch loop
     void process( generic_scheduler& );
 
     //! Notification that worker or master leaves its arena
-    inline void on_thread_leaving ();
+    template<bool is_master>
+    inline void on_thread_leaving ( );
 
 #if __TBB_STATISTICS
     //! Outputs internal statistics accumulated by the arena
@@ -270,12 +294,20 @@ private:
 
 #include "market.h"
 #include "scheduler_common.h"
+#include "governor.h"
 
 namespace tbb {
 namespace internal {
 
-inline void arena::on_thread_leaving () {
-    // In case of using fire-and-forget tasks (scheduled via task::enqueue()) 
+template<bool is_master>
+inline void arena::on_thread_leaving ( ) {
+    //
+    // Implementation of arena destruction synchronization logic contained various
+    // bugs/flaws at the different stages of its evolution, so below is a detailed
+    // description of the issues taken into consideration in the framework of the
+    // current design.
+    //
+    // In case of using fire-and-forget tasks (scheduled via task::enqueue())
     // master thread is allowed to leave its arena before all its work is executed,
     // and market may temporarily revoke all workers from this arena. Since revoked
     // workers never attempt to reset arena state to EMPTY and cancel its request
@@ -283,31 +315,55 @@ inline void arena::on_thread_leaving () {
     // thread is leaving it and arena's state is EMPTY (that is its master thread
     // left and it does not contain any work).
     //
-    // A worker that checks for work presence and transitions arena to the EMPTY 
+    // A worker that checks for work presence and transitions arena to the EMPTY
     // state (in snapshot taking procedure arena::is_out_of_work()) updates
     // arena::my_pool_state first and only then arena::my_num_workers_requested.
-    // So the below check for work absence must be done against the latter field.
+    // So the check for work absence must be done against the latter field.
     //
-    // Besides there is a time window between decrementing the active threads count
-    // and checking if there is an outstanding request for workers. New worker 
-    // thread may arrive during this window, finish whatever work is present, and
-    // then shutdown the arena. This sequence may result in destructing the same
-    // arena twice. So a local copy of the outstanding request value must be done
-    // before decrementing active threads count.
+    // In a time window between decrementing the active threads count and checking
+    // if there is an outstanding request for workers. New worker thread may arrive,
+    // finish remaining work, set arena state to empty, and leave decrementing its
+    // refcount and destroying. Then the current thread will destroy the arena
+    // the second time. To preclude it a local copy of the outstanding request
+    // value can be stored before decrementing active threads count.
     //
-    int requested = __TBB_load_with_acquire(my_num_workers_requested);
-    if ( --my_num_threads_active==0 && !requested ) {
-        __TBB_ASSERT( !my_num_workers_requested, NULL );
-        __TBB_ASSERT( my_pool_state == SNAPSHOT_EMPTY || !my_max_num_workers, NULL );
-        close_arena();
-    }
+    // But this technique may cause two other problem. When the stored request is
+    // zero, it is possible that arena still has threads and they can generate new
+    // tasks and thus re-establish non-zero requests. Then all the threads can be
+    // revoked (as described above) leaving this thread the last one, and causing
+    // it to destroy non-empty arena.
+    //
+    // The other problem takes place when the stored request is non-zero. Another
+    // thread may complete the work, set arena state to empty, and leave without
+    // arena destruction before this thread decrements the refcount. This thread
+    // cannot destroy the arena either. Thus the arena may be "orphaned".
+    //
+    // In both cases we cannot dereference arena pointer after the refcount is
+    // decremented, as our arena may already be destroyed.
+    //
+    // If this is the master thread, market can be concurrently destroyed.
+    // In case of workers market's liveness is ensured by the RML connection
+    // rundown protocol, according to which the client (i.e. the market) lives
+    // until RML server notifies it about connection termination, and this
+    // notification is fired only after all workers return into RML.
+    //
+    // Thus if we decremented refcount to zero we ask the market to check arena
+    // state (including the fact if it is alive) under the lock.
+    //
+    uintptr_t aba_epoch = my_aba_epoch;
+    market* m = my_market;
+    __TBB_ASSERT(my_references > int(!is_master), "broken arena reference counter");
+    if ( (my_references -= is_master? 1:2 ) == 0 ) // worker's counter starts from bit 1
+        market::try_destroy_arena( m, this, aba_epoch, is_master );
 }
 
 template<bool Spawned> void arena::advertise_new_work() {
     if( !Spawned ) { // i.e. the work was enqueued
         if( my_max_num_workers==0 ) {
             my_max_num_workers = 1;
+            __TBB_ASSERT(!my_mandatory_concurrency, "");
             my_mandatory_concurrency = true;
+            __TBB_ASSERT(!num_workers_active(), "");
             my_pool_state = SNAPSHOT_FULL;
             my_market->adjust_demand( *this, 1 );
             return;
@@ -342,6 +398,7 @@ template<bool Spawned> void arena::advertise_new_work() {
             if( Spawned ) {
                 if( my_mandatory_concurrency ) {
                     __TBB_ASSERT(my_max_num_workers==1, "");
+                    __TBB_ASSERT(!governor::local_scheduler()->is_worker(), "");
                     // There was deliberate oversubscription on 1 core for sake of starvation-resistant tasks.
                     // Now a single active thread (must be the master) supposedly starts a new parallel region
                     // with relaxed sequential semantics, and oversubscription should be avoided.
