@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -31,16 +31,28 @@
 namespace tbb {
 namespace internal {
 
-void concurrent_monitor::prepare_wait( thread_context& thr, void* ctx ) {
+void concurrent_monitor::thread_context::init() {
+    new (sema.begin()) binary_semaphore;
+    ready = true;
+}
+
+concurrent_monitor::~concurrent_monitor() {
+    abort_all();
+    __TBB_ASSERT( waitset_ec.empty(), "waitset not empty?" );
+}
+
+void concurrent_monitor::prepare_wait( thread_context& thr, uintptr_t ctx ) {
+    if( !thr.ready )
+        thr.init();
     // this is good place to pump previous spurious wakeup
-    if( thr.spurious ) {
+    else if( thr.spurious ) {
         thr.spurious = false;
-        thr.sema.P();
+        thr.semaphore().P();
     }
     thr.context = ctx;
     thr.in_waitset = true;
     {
-        spin_mutex::scoped_lock l( mutex_ec );
+        tbb::spin_mutex::scoped_lock l( mutex_ec );
         __TBB_store_relaxed( thr.epoch, __TBB_load_relaxed(epoch) );
         waitset_ec.add( (waitset_t::node_t*)&thr );
     }
@@ -79,7 +91,7 @@ void concurrent_monitor::notify_one_relaxed() {
         }
     }
     if( n!=end )
-        to_thread_context(n)->sema.V();
+        to_thread_context(n)->semaphore().V();
 }
 
 void concurrent_monitor::notify_all_relaxed() {
@@ -98,9 +110,33 @@ void concurrent_monitor::notify_all_relaxed() {
     waitset_node_t* nxt;
     for( waitset_node_t* n=temp.front(); n!=end; n=nxt ) {
         nxt = n->next;
-        to_thread_context(n)->sema.V();
+        to_thread_context(n)->semaphore().V();
     }
-#if TBB_USE_DEBUG
+#if TBB_USE_ASSERT
+    temp.clear();
+#endif
+}
+
+void concurrent_monitor::abort_all_relaxed() {
+    if( waitset_ec.empty() )
+        return;
+    dllist_t temp;
+    const waitset_node_t* end;
+    {
+        tbb::spin_mutex::scoped_lock l( mutex_ec );
+        __TBB_store_relaxed( epoch, __TBB_load_relaxed(epoch) + 1 );
+        waitset_ec.flush_to( temp );
+        end = temp.end();
+        for( waitset_node_t* n=temp.front(); n!=end; n=n->next )
+            to_thread_context(n)->in_waitset = false;
+    }
+    waitset_node_t* nxt;
+    for( waitset_node_t* n=temp.front(); n!=end; n=nxt ) {
+        nxt = n->next;
+        to_thread_context(n)->aborted = true;
+        to_thread_context(n)->semaphore().V();
+    }
+#if TBB_USE_ASSERT
     temp.clear();
 #endif
 }
