@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -32,6 +32,8 @@
 #include "tbb/tbb_stddef.h"
 #include "tbb/atomic.h"
 #include "tbb/spin_mutex.h"
+#include "tbb/tbb_exception.h"
+#include "tbb/aligned_space.h"
 
 #include "semaphore.h"
 
@@ -88,20 +90,16 @@ public:
         }
     }
 
-#if !TBB_USE_DEBUG
+    void clear() {head.next = head.prev = &head; __TBB_store_relaxed(count, 0);}
 private:
-#endif
     __TBB_atomic size_t count;
     node_t head;
-    void clear() {head.next = &head; head.prev = &head;__TBB_store_relaxed(count, 0);}
 };
 
 typedef circular_doubly_linked_list_with_sentinel waitset_t;
 typedef circular_doubly_linked_list_with_sentinel dllist_t;
 typedef circular_doubly_linked_list_with_sentinel::node_t waitset_node_t;
 
-class concurrent_monitor;
-
 //! concurrent_monitor
 /** fine-grained concurrent_monitor implementation */
 class concurrent_monitor : no_copy {
@@ -110,21 +108,39 @@ public:
     class thread_context : waitset_node_t, no_copy {
         friend class concurrent_monitor;
     public:
-        thread_context() : spurious(false), context(NULL) {epoch = 0; in_waitset = false;}
-        ~thread_context() { if( spurious ) sema.P(); }
+        thread_context() : spurious(false), aborted(false), ready(false), context(0) {
+            epoch = 0;
+            in_waitset = false;
+        }
+        ~thread_context() {
+            if (ready) {
+                if( spurious ) semaphore().P();
+                semaphore().~binary_semaphore();
+            }
+        }
+        binary_semaphore& semaphore() { return *sema.begin(); }
     private:
-        binary_semaphore   sema;
+        //! The method for lazy initialization of the thread_context's semaphore.
+        //  Inlining of the method is undesirable, due to extra instructions for
+        //  exception support added at caller side.
+        __TBB_NOINLINE( void init() );
+        tbb::aligned_space<binary_semaphore, 1> sema;
         __TBB_atomic unsigned epoch;
-        tbb::atomic<bool>     in_waitset;
-        bool         spurious;
-        void*        context;
+        tbb::atomic<bool> in_waitset;
+        bool  spurious;
+        bool  aborted;
+        bool  ready;
+        uintptr_t context;
     };
 
     //! ctor
     concurrent_monitor() {__TBB_store_relaxed(epoch, 0);}
 
+    //! dtor
+    ~concurrent_monitor() ; 
+
     //! prepare wait by inserting 'thr' into the wailt queue
-    void prepare_wait( thread_context& thr, void* ctx = 0 );
+    void prepare_wait( thread_context& thr, uintptr_t ctx = 0 );
 
     //! Commit wait if event count has not changed; otherwise, cancel wait.
     /** Returns true if committed, false if canceled. */
@@ -132,8 +148,11 @@ public:
         const bool do_it = thr.epoch == __TBB_load_relaxed(epoch);
         // this check is just an optimization
         if( do_it ) {
-            thr.sema.P();
+            __TBB_ASSERT( thr.ready, "use of commit_wait() without prior prepare_wait()");
+            thr.semaphore().P();
             __TBB_ASSERT( !thr.in_waitset, "still in the queue?" );
+            if( thr.aborted )
+                throw_exception( eid_user_abort );
         } else {
             cancel_wait( thr );
         }
@@ -142,6 +161,10 @@ public:
     //! Cancel the wait. Removes the thread from the wait queue if not removed yet.
     void cancel_wait( thread_context& thr );
 
+    //! Wait for a condition to be satisfied with waiting-on context
+    template<typename WaitUntil, typename Context>
+    void wait( WaitUntil until, Context on );
+
     //! Notify one thread about the event
     void notify_one() {atomic_fence(); notify_one_relaxed();}
 
@@ -160,6 +183,12 @@ public:
     //! Notify waiting threads of the event that satisfies the given predicate; Relaxed version
     template<typename P> void notify_relaxed( const P& predicate );
 
+    //! Abort any sleeping threads at the time of the call
+    void abort_all() {atomic_fence(); abort_all_relaxed(); }
+ 
+    //! Abort any sleeping threads at the time of the call; Relaxed version
+    void abort_all_relaxed();
+
 private:
     tbb::spin_mutex mutex_ec;
     waitset_t       waitset_ec;
@@ -167,6 +196,22 @@ private:
     thread_context* to_thread_context( waitset_node_t* n ) { return static_cast<thread_context*>(n); }
 };
 
+template<typename WaitUntil, typename Context>
+void concurrent_monitor::wait( WaitUntil until, Context on )
+{
+    bool slept = false;
+    thread_context thr_ctx;
+    prepare_wait( thr_ctx, on() );
+    while( !until() ) {
+        if( (slept = commit_wait( thr_ctx ) )==true )
+            if( until() ) break;
+        slept = false;
+        prepare_wait( thr_ctx, on() );
+    }
+    if( !slept )
+        cancel_wait( thr_ctx );
+}
+
 template<typename P>
 void concurrent_monitor::notify_relaxed( const P& predicate ) {
         if( waitset_ec.empty() )
@@ -191,9 +236,9 @@ void concurrent_monitor::notify_relaxed( const P& predicate ) {
         end = temp.end();
         for( waitset_node_t* n=temp.front(); n!=end; n=nxt ) {
             nxt = n->next;
-            to_thread_context(n)->sema.V();
+            to_thread_context(n)->semaphore().V();
         }
-#if TBB_USE_DEBUG
+#if TBB_USE_ASSERT
         temp.clear();
 #endif
 }
