@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -39,9 +39,12 @@
     #endif
 #endif /* WIN */
 
+#ifndef INTEL_ITTNOTIFY_API_PRIVATE
+#define INTEL_ITTNOTIFY_API_PRIVATE
+#endif
+
 #include "tools_api/ittnotify.h"
 #include "tools_api/legacy/ittnotify.h"
-#include "tools_api/internal/ittnotify.h"
 
 #if _WIN32||_WIN64
     #undef _T
