@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -93,7 +93,7 @@ public:
             m_size += m_cur_segment_size;
             m_cur_segment_size *= 2;
             m_pos = m_cur_segment_size;
-            m_segments[m_num_segments++] = m_cur_segment = (T*)NFS_Allocate( m_cur_segment_size * sizeof(T), 1, NULL );
+            m_segments[m_num_segments++] = m_cur_segment = (T*)NFS_Allocate( m_cur_segment_size, sizeof(T), NULL );
             __TBB_ASSERT ( m_num_segments < max_segments, "Maximal capacity exceeded" );
         }
         m_cur_segment[--m_pos] = val;
@@ -103,14 +103,14 @@ public:
     /** Can only be used when T is a POD type, as copying does not invoke copy constructors. **/
     void copy_memory ( T* dst ) const
     {
-        size_t size = m_cur_segment_size - m_pos;
-        memcpy( dst, m_cur_segment + m_pos, size * sizeof(T) );
-        dst += size;
-        size = m_cur_segment_size / 2;
+        size_t sz = m_cur_segment_size - m_pos;
+        memcpy( dst, m_cur_segment + m_pos, sz * sizeof(T) );
+        dst += sz;
+        sz = m_cur_segment_size / 2;
         for ( long i = (long)m_num_segments - 2; i >= 0; --i ) {
-            memcpy( dst, m_segments[i], size * sizeof(T) );
-            dst += size;
-            size /= 2;
+            memcpy( dst, m_segments[i], sz * sizeof(T) );
+            dst += sz;
+            sz /= 2;
         }
     }
 
