@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -28,6 +28,7 @@
 
 #include "tbb/spin_rw_mutex.h"
 #include "tbb/tbb_machine.h"
+#include "tbb/atomic.h"
 #include "itt_notify.h"
 
 #if defined(_MSC_VER) && defined(_Wp64)
@@ -39,17 +40,16 @@ namespace tbb {
 
 template<typename T> // a template can work with private spin_rw_mutex::state_t
 static inline T CAS(volatile T &addr, T newv, T oldv) {
-    // ICC (9.1 and 10.1 tried) unable to do implicit conversion 
+    // ICC (9.1 and 10.1 tried) unable to do implicit conversion
     // from "volatile T*" to "volatile void*", so explicit cast added.
-    return T(__TBB_CompareAndSwapW((volatile void *)&addr, (intptr_t)newv, (intptr_t)oldv));
+    return tbb::internal::as_atomic(addr).compare_and_swap( newv, oldv );
 }
 
 //! Acquire write lock on the given mutex.
 bool spin_rw_mutex_v3::internal_acquire_writer()
 {
     ITT_NOTIFY(sync_prepare, this);
-    internal::atomic_backoff backoff;
-    for(;;) {
+    for( internal::atomic_backoff backoff;;backoff.pause() ){
         state_t s = const_cast<volatile state_t&>(state); // ensure reloading
         if( !(s & BUSY) ) { // no readers, no writers
             if( CAS(state, WRITER, s)==s )
@@ -58,25 +58,23 @@ bool spin_rw_mutex_v3::internal_acquire_writer()
         } else if( !(s & WRITER_PENDING) ) { // no pending writers
             __TBB_AtomicOR(&state, WRITER_PENDING);
         }
-        backoff.pause();
     }
     ITT_NOTIFY(sync_acquired, this);
     return false;
 }
 
 //! Release writer lock on the given mutex
-void spin_rw_mutex_v3::internal_release_writer() 
+void spin_rw_mutex_v3::internal_release_writer()
 {
     ITT_NOTIFY(sync_releasing, this);
     __TBB_AtomicAND( &state, READERS );
 }
 
 //! Acquire read lock on given mutex.
-void spin_rw_mutex_v3::internal_acquire_reader() 
+void spin_rw_mutex_v3::internal_acquire_reader()
 {
     ITT_NOTIFY(sync_prepare, this);
-    internal::atomic_backoff backoff;
-    for(;;) {
+    for( internal::atomic_backoff b;;b.pause() ){
         state_t s = const_cast<volatile state_t&>(state); // ensure reloading
         if( !(s & (WRITER|WRITER_PENDING)) ) { // no writer or write requests
             state_t t = (state_t)__TBB_FetchAndAddW( &state, (intptr_t) ONE_READER );
@@ -85,7 +83,6 @@ void spin_rw_mutex_v3::internal_acquire_reader()
             // writer got there first, undo the increment
             __TBB_FetchAndAddW( &state, -(intptr_t)ONE_READER );
         }
-        backoff.pause();
     }
 
     ITT_NOTIFY(sync_acquired, this);
@@ -93,8 +90,8 @@ void spin_rw_mutex_v3::internal_acquire_reader()
 }
 
 //! Upgrade reader to become a writer.
-/** Returns true if the upgrade happened without re-acquiring the lock and false if opposite */
-bool spin_rw_mutex_v3::internal_upgrade() 
+/** Returns whether the upgrade happened without releasing and re-acquiring the lock */
+bool spin_rw_mutex_v3::internal_upgrade()
 {
     state_t s = state;
     __TBB_ASSERT( s & READERS, "invalid state before upgrade: no readers " );
@@ -104,14 +101,11 @@ bool spin_rw_mutex_v3::internal_upgrade()
     while( (s & READERS)==ONE_READER || !(s & WRITER_PENDING) ) {
         state_t old_s = s;
         if( (s=CAS(state, s | WRITER | WRITER_PENDING, s))==old_s ) {
-            internal::atomic_backoff backoff;
             ITT_NOTIFY(sync_prepare, this);
-            // the state should be 0...0111, i.e. 1 reader and waiting writer;
-            // both new readers and writers are blocked
-            while( (state & READERS) != ONE_READER ) // more than 1 reader
-                backoff.pause(); 
+            internal::atomic_backoff backoff;
+            while( (state & READERS) != ONE_READER ) backoff.pause();
             __TBB_ASSERT((state&(WRITER_PENDING|WRITER))==(WRITER_PENDING|WRITER),"invalid state when upgrading to writer");
-
+            // both new readers and writers are blocked at this time
             __TBB_FetchAndAddW( &state,  - (intptr_t)(ONE_READER+WRITER_PENDING));
             ITT_NOTIFY(sync_acquired, this);
             return true; // successfully upgraded
@@ -167,7 +161,6 @@ bool spin_rw_mutex_v3::internal_try_acquire_reader()
     return false;
 }
 
-
 void spin_rw_mutex_v3::internal_construct() {
     ITT_SYNC_CREATE(this, _T("tbb::spin_rw_mutex"), _T(""));
 }
