@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -38,7 +38,6 @@
 #include <stdarg.h>
 #if _MSC_VER
 #include <crtdbg.h>
-#define __TBB_USE_DBGBREAK_DLG TBB_USE_DEBUG
 #endif
 
 #if _MSC_VER >= 1400
@@ -49,7 +48,11 @@
 
 using namespace std;
 
+#if __TBBMALLOC_BUILD
+namespace rml { namespace internal {
+#else
 namespace tbb {
+#endif
     //! Type for an assertion handler
     typedef void(*assertion_handler_type)( const char* filename, int line, const char* expression, const char * comment );
 
@@ -72,7 +75,7 @@ namespace tbb {
                          expression, line, filename );
                 if( comment )
                     fprintf( stderr, "Detailed description: %s\n", comment );
-#if __TBB_USE_DBGBREAK_DLG
+#if _MSC_VER && _DEBUG
                 if(1 == _CrtDbgReport(_CRT_ASSERT, filename, line, "tbb_debug.dll", "%s\r\n%s", expression, comment?comment:""))
                         _CrtDbgBreak();
 #else
@@ -87,6 +90,7 @@ namespace tbb {
 #   define vsnprintf _vsnprintf
 #endif
 
+#if !__TBBMALLOC_BUILD
     namespace internal {
         //! Report a runtime warning.
         void __TBB_EXPORTED_FUNC runtime_warning( const char* format, ... )
@@ -94,8 +98,14 @@ namespace tbb {
             char str[1024]; memset(str, 0, 1024);
             va_list args; va_start(args, format);
             vsnprintf( str, 1024-1, format, args);
+            va_end(args);
             fprintf( stderr, "TBB Warning: %s\n", str);
         }
     } // namespace internal
+#endif
 
-} /* namespace tbb */
+#if __TBBMALLOC_BUILD
+}} // namespaces rml::internal
+#else
+}  // namespace tbb
+#endif
