@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -38,7 +38,9 @@
 // Enable preview features if any 
 #define __TBB_BUILD 1
 
-#define __TBB_TASK_CPP_DIRECTLY_INCLUDED 1
+#undef DO_ITT_NOTIFY
+
+#define __TBB_SOURCE_DIRECTLY_INCLUDED 1
 #include "../tbb/tbb_main.cpp"
 #include "../tbb/dynamic_link.cpp"
 #include "../tbb/tbb_misc_ex.cpp"
@@ -59,7 +61,22 @@
 #include "../tbb/spin_rw_mutex.cpp"
 #include "../tbb/spin_mutex.cpp"
 #include "../tbb/private_server.cpp"
+#include "../tbb/concurrent_monitor.cpp"
+#if _WIN32||_WIN64
 #include "../tbb/semaphore.cpp"
+#endif
 #include "../rml/client/rml_tbb.cpp"
 
+#if HARNESS_USE_RUNTIME_LOADER
+#undef HARNESS_USE_RUNTIME_LOADER
+#include "harness.h"
+
+int TestMain () {
+    // Tests that directly include sources make no sense in runtime loader testing mode.
+    return Harness::Skipped;
+}
+// Renaming the TestMain function avoids conditional compilation around same function in the test file
+#define TestMain TestMainSkipped
+#endif
+
 #endif /* harness_inject_scheduler_H */
