@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -32,7 +32,7 @@
 #define TBB_PREVIEW_MEMORY_POOL 1
 
 #include "harness_assert.h"
-#if __linux__  && __ia64__
+#if !__TBB_SOURCE_DIRECTLY_INCLUDED
 // Currently pools high-level interface has dependency to TBB library
 // to get atomics. For sake of testing add rudementary implementation of them.
 #include "harness_tbb_independence.h"
@@ -40,6 +40,7 @@
 #include "tbb/memory_pool.h"
 #include "tbb/scalable_allocator.h"
 
+#define HARNESS_TBBMALLOC_THREAD_SHUTDOWN 1
 // the actual body of the test is there:
 #include "test_allocator.h"
 #include "harness_allocator.h"
@@ -81,8 +82,110 @@ public:
     }
 };
 
+#if TBB_USE_EXCEPTIONS
+
+class NullAllocator {
+public:
+    typedef char value_type;
+    NullAllocator() { }
+    NullAllocator(const NullAllocator&) { }
+    ~NullAllocator() { }
+    void *allocate(size_t) { return NULL; }
+    void deallocate(void *, size_t) { ASSERT(0, NULL); }
+};
+
+void TestZeroSpaceMemoryPool()
+{
+    try {
+        tbb::memory_pool<NullAllocator> pool;
+        ASSERT(0, "Useless allocator with no memory must not be created");
+    } catch (std::bad_alloc) {
+    } catch (...) {
+        ASSERT(0, "wrong exception type; expected bad_alloc");
+    }
+}
+
+#else // TBB_USE_EXCEPTIONS
+
+void TestZeroSpaceMemoryPool() { }
+
+struct FixedPool {
+    void  *buf;
+    size_t size;
+    bool   used;
+    FixedPool(void *buf, size_t size) : buf(buf), size(size), used(false) {}
+};
+
+static void *fixedBufGetMem(intptr_t pool_id, size_t &bytes)
+{
+    if (((FixedPool*)pool_id)->used)
+        return NULL;
+
+    ((FixedPool*)pool_id)->used = true;
+    bytes = ((FixedPool*)pool_id)->size;
+    return ((FixedPool*)pool_id)->buf;
+}
+
+#endif // TBB_USE_EXCEPTIONS
+
+/* test that pools in small space are either usable or not created
+   (i.e., exception raised) */
+void TestSmallFixedSizePool()
+{
+    char *buf;
+    bool allocated = false;
+
+    for (size_t sz = 0; sz < 64*1024; sz = sz? 3*sz : 3) {
+        buf = (char*)malloc(sz);
+#if TBB_USE_EXCEPTIONS
+        try {
+            tbb::fixed_pool pool(buf, sz);
+/* Check that pool is usable, i.e. such an allocation exists,
+   that can be fulfilled from the pool. 16B allocation fits in 16KB slabs,
+   so it requires at least 16KB. Requirement of 9KB allocation is more modest.
+*/
+            allocated = pool.malloc( 16 ) || pool.malloc( 9*1024 );
+            ASSERT(allocated, "If pool created, it must be useful.");
+        } catch (std::bad_alloc) {
+        } catch (...) {
+            ASSERT(0, "wrong exception type; expected bad_alloc");
+        }
+#else
+/* Do not test high-level pool interface because pool ctor emit exception
+   on creation failure. Instead test same functionality via low-level interface.
+   TODO: add support for configuration with disabled exceptions to pools.
+*/
+        rml::MemPoolPolicy pol(fixedBufGetMem, NULL, 0, /*fixedSizePool=*/true,
+                               /*keepMemTillDestroy=*/false);
+        rml::MemoryPool *pool;
+        FixedPool fixedPool(buf, sz);
+
+        rml::MemPoolError ret = pool_create_v1((intptr_t)&fixedPool, &pol, &pool);
+
+        if (ret == rml::POOL_OK) {
+            allocated = pool_malloc(pool, 16) || pool_malloc(pool, 9*1024);
+            ASSERT(allocated, "If pool created, it must be useful.");
+            pool_destroy(pool);
+        } else
+            ASSERT(ret == rml::NO_MEMORY, "Expected that pool either valid "
+                                     "or have no memory to be created");
+#endif
+        free(buf);
+    }
+    ASSERT(allocated, "Maximal buf size should be enough to create working fixed_pool");
+#if TBB_USE_EXCEPTIONS
+    try {
+        tbb::fixed_pool pool(NULL, 10*1024*1024);
+        ASSERT(0, "Useless allocator with no memory must not be created");
+    } catch (std::bad_alloc) {
+    } catch (...) {
+        ASSERT(0, "wrong exception type; expected bad_alloc");
+    }
+#endif
+}
+
 int TestMain () {
-#if _MSC_VER && !__TBBMALLOC_NO_IMPLICIT_LINKAGE
+#if _MSC_VER && !__TBBMALLOC_NO_IMPLICIT_LINKAGE && !__TBB_WIN8UI_SUPPORT
     #ifdef _DEBUG
         ASSERT(!GetModuleHandle("tbbmalloc.dll") && GetModuleHandle("tbbmalloc_debug.dll"),
             "test linked with wrong (non-debug) tbbmalloc library");
@@ -108,9 +211,9 @@ int TestMain () {
         strcpy(p1, text);
         char *p2 = (char*)pool.realloc( p1, 15 );
         ASSERT( p2 && !strcmp(p2, text), "realloc broke memory" );
-        
+
         result += TestMain(tbb::memory_pool_allocator<void>(pool) );
-        
+
         // try allocate almost entire buf keeping some reasonable space for internals
         char *p3 = (char*)pool.realloc( p2, sizeof(buf)-128*1024 );
         ASSERT( p3, "defragmentation failed" );
@@ -122,6 +225,9 @@ int TestMain () {
 
         result += TestMain(tbb::memory_pool_allocator<void>(pool) );
     }
+    TestSmallFixedSizePool();
+    TestZeroSpaceMemoryPool();
+
     ASSERT( !result, NULL );
     return Harness::Done;
 }
