@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -190,6 +190,9 @@ struct Body: NoAssign {
         for( size_t k=0; k<256; ++k )
             if(array[k])
                 check_deallocate(array, k, thread_id);
+#if __TBBMALLOC_CALL_THREAD_SHUTDOWN
+        __TBB_mallocThreadShutdownNotification();
+#endif
     }
 };
 
