@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -40,6 +40,8 @@ void TestSequence(const typename Container::allocator_type &a) {
         ASSERT( *p==i*i, NULL );
         ++p;
     }
+    // regression test against compilation error for GCC 4.6.2
+    c.resize(1000);
 }
 
 template<typename Set>
@@ -82,9 +84,11 @@ void TestMap(const typename Map::allocator_type &a) {
 template<typename Allocator>
 void TestAllocatorWithSTL(const Allocator &a = Allocator() ) {
     typedef typename Allocator::template rebind<int>::other Ai;
-    typedef typename Allocator::template rebind<const int>::other Aci;
     typedef typename Allocator::template rebind<std::pair<const int, int> >::other Acii;
+#if _MSC_VER
+    typedef typename Allocator::template rebind<const int>::other Aci;
     typedef typename Allocator::template rebind<std::pair<int, int> >::other Aii;
+#endif
 
     // Sequenced containers
     TestSequence<std::deque <int,Ai> >(a);
