@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -27,6 +27,17 @@
 */
 
 // Program for basic correctness testing of assembly-language routines.
+#include "harness_defs.h"
+//for ICC builtins mode the test will be skipped as
+//macro __TBB_GCC_BUILTIN_ATOMICS_PRESENT used to define __TBB_TEST_SKIP_GCC_BUILTINS_MODE
+//will not be defined (it is explicitly disabled for ICC)
+#if __TBB_TEST_SKIP_GCC_BUILTINS_MODE
+#include "harness.h"
+int TestMain() {
+    REPORT("Known issue: GCC builtins aren't available\n");
+    return Harness::Skipped;
+}
+#else
 
 #include "tbb/task.h"
 
@@ -35,6 +46,7 @@
 
 using tbb::internal::reference_count;
 
+//TODO: remove this function when atomic function __TBB_XXX are dropped
 //! Test __TBB_CompareAndSwapW
 static void TestCompareExchange() {
     ASSERT( intptr_t(-10)<10, "intptr_t not a signed integral type?" ); 
@@ -58,6 +70,7 @@ static void TestCompareExchange() {
             }
 }
 
+//TODO: remove this function when atomic function __TBB_XXX are dropped
 //! Test __TBB___TBB_FetchAndIncrement and __TBB___TBB_FetchAndDecrement
 static void TestAtomicCounter() {
     // "canary" is a value used to detect illegal overwrites.
@@ -99,12 +112,12 @@ static void TestTinyLock() {
 #endif
     __TBB_LockByte(flags[8]);
     for( unsigned int i=0; i<16; ++i )
-	#ifdef __sparc
+        #ifdef __sparc
         ASSERT( flags[i]==(i==8?0xff:i), NULL );
-	#else
+        #else
         ASSERT( flags[i]==(i==8?1:i), NULL );
-	#endif
-    __TBB_UnlockByte(flags[8], 0);
+        #endif
+    __TBB_UnlockByte(flags[8]);
     for( unsigned int i=0; i<16; ++i )
         ASSERT( flags[i] == (i==8?0:i), NULL );
 }
@@ -140,3 +153,4 @@ int TestMain () {
     }
     return Harness::Done;
 }
+#endif // __TBB_TEST_SKIP_BUILTINS_MODE
