@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -401,11 +401,16 @@ void TestConstExprInitializationIsTranslationTime(){
     ASSERT(a == 8,ct_init_failed_msg);
 
     constexpr tbb::atomic<test_constexpr_initialization_helper::white_box_ad_hoc_type> ct_atomic(10);
+    //for some unknown reason clang does not managed to enum syntax
+#if __clang__
+    constexpr int ct_atomic_value_ten = (int)ct_atomic;
+#else
     enum {ct_atomic_value_ten = (int)ct_atomic};
+#endif
     __TBB_STATIC_ASSERT(ct_atomic_value_ten == 10, "translation time init failed?");
     ASSERT(ct_atomic_value_ten == 10,ct_init_failed_msg);
     int array[ct_atomic_value_ten];
-    ASSERT(array_length(array) == 10,ct_init_failed_msg);
+    ASSERT(Harness::array_length(array) == 10,ct_init_failed_msg);
 }
 
 #include <string>
@@ -450,7 +455,7 @@ namespace TestConstExprInitializationOfGlobalObjectsHelper{
 
         template <typename T>
         struct registration{
-            registration(){const_expr_tests.push_back(TestStaticInit<T>);}
+            registration(){const_expr_tests.push_back(&TestStaticInit<T>);}
         };
     }
     //according to ISO C++11 [basic.start.init], static data fields of class template have unordered
