@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -28,6 +28,7 @@
 
 #include "harness.h"
 #include "tbb/flow_graph.h"
+#include "tbb/task.h"
 
 #include "tbb/atomic.h"
 
@@ -63,17 +64,20 @@ public:
         return v;
     }
 
-    /* override */ bool try_put( const T &v ) {
+    /* override */ tbb::task * try_put_task( const T &v ) {
         ++my_counters[(int)v];
-        return true;
+        return const_cast<tbb::task *>(tbb::flow::interface6::SUCCESSFULLY_ENQUEUED);
     }
 
+    /*override*/void reset_receiver() { }
+
 };
 
 template< typename T >
 void test_serial_broadcasts() {
 
-    tbb::flow::broadcast_node<T> b;
+    tbb::flow::graph g;
+    tbb::flow::broadcast_node<T> b(g);
     
     for ( int num_receivers = 1; num_receivers < R; ++num_receivers ) {
         counting_array_receiver<T> *receivers = new counting_array_receiver<T>[num_receivers];
@@ -104,8 +108,8 @@ void test_serial_broadcasts() {
 
 template< typename T >
 class native_body : private NoAssign {
- 
-   tbb::flow::broadcast_node<T> &my_b;
+
+    tbb::flow::broadcast_node<T> &my_b;
 
 public:
 
@@ -148,7 +152,8 @@ void run_parallel_broadcasts(int p, tbb::flow::broadcast_node<T>& b) {
 template< typename T >
 void test_parallel_broadcasts(int p) {
 
-    tbb::flow::broadcast_node<T> b;
+    tbb::flow::graph g;
+    tbb::flow::broadcast_node<T> b(g);
     run_parallel_broadcasts(p, b);
     
     // test copy constructor
