@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -34,16 +34,14 @@
 #define CILK_SYMBOLS_VISIBLE (_WIN32||_WIN64)
 // The compiler does not add "-lcilkrts" linker option on some linux systems
 #define CILK_LINKAGE_BROKEN  (__linux__ && __GNUC__<4 && __INTEL_COMPILER_BUILD_DATE <= 20110427)
-// In U4, cilkrts incorrectly sends the interop notifications to TBB
-#define CILK_NOTIFICATIONS_BROKEN ( __INTEL_COMPILER_BUILD_DATE == 20110427 )
+// Currently, the interop doesn't support the situation:
+//1) Intel(R) Threading Building Blocks (Intel(R) TBB) is outermost;
+//2)   Intel(R) Cilk(TM) Plus, and it should be dynamically loaded with dlopen/LoadLibrary (possibly via a 3rd party module);
+//3)     Intel(R) TBB again;
+//4)       Intel(R) Cilk(TM) Plus again.
+#define HEAVY_NESTED_INTEROP_SUPPORT ( __INTEL_COMPILER_BUILD_DATE < 20110427 )
 
-#if __TBB_CILK_INTEROP && CILK_SYMBOLS_VISIBLE && !CILK_LINKAGE_BROKEN && !CILK_NOTIFICATIONS_BROKEN
-
-#if _WIN32 || _WIN64
-#include "tbb/machine/windows_api.h"
-#else
-#include <dlfcn.h>
-#endif /* _WIN32 || _WIN64 */
+#if __TBB_CILK_INTEROP && CILK_SYMBOLS_VISIBLE && !CILK_LINKAGE_BROKEN && HEAVY_NESTED_INTEROP_SUPPORT
 
 #include "tbb/task_scheduler_init.h"
 #include "tbb/task.h"
@@ -81,6 +79,7 @@ CILK_TEST_EXPORT void CilkShutdown()
 #else /* _USRDLL undefined */
 
 #include "harness.h"
+#include "harness_dynamic_libs.h"
 
 int SerialFib( int n ) {
     int a=0, b=1;
@@ -107,30 +106,11 @@ class FibTask: public tbb::task {
             result = n;
         } else {
 
-#if _WIN32 || _WIN64
-            HMODULE hLib = LoadLibrary("test_cilk_dynamic_load_dll.dll");
-            ASSERT( hLib, "failed to load test_cilk_dynamic_load_dll" );
-
-            CilkFib = (CILK_CALL) GetProcAddress(hLib, "CilkFib");
-            ASSERT( CilkFib, "failed to get address" );
-
-            CilkShutdown = (CILK_SHUTDOWN) GetProcAddress(hLib, "CilkShutdown");
-            ASSERT( CilkShutdown, "failed to get address" );
-#else /* !WIN */
-        #if __APPLE__
-            #define LIBRARY_NAME(base) base".dylib"
-        #else
-            #define LIBRARY_NAME(base) base".so"
-        #endif /* __APPLE__ */
-            void *hLib = dlopen( LIBRARY_NAME("test_cilk_dynamic_load_dll"), RTLD_LAZY );
-            ASSERT( hLib, "failed to load test_cilk_dynamic_load_dll" );
-
-            CilkFib = (CILK_CALL) dlsym(hLib, "CilkFib");
-            ASSERT( CilkFib, "failed to get address" );
-
-            CilkShutdown = (CILK_SHUTDOWN) dlsym(hLib, "CilkShutdown");
-            ASSERT( CilkShutdown, "failed to get address" );
-#endif /* !WIN */
+            // TODO: why RTLD_LAZY was used here?
+            Harness::LIBRARY_HANDLE hLib =
+                Harness::OpenLibrary(TEST_LIBRARY_NAME("test_cilk_dynamic_load_dll"));
+            CilkFib = (CILK_CALL)Harness::GetAddress(hLib, "CilkFib");
+            CilkShutdown = (CILK_SHUTDOWN)Harness::GetAddress(hLib, "CilkShutdown");
 
             int x, y;
             x = CilkFib(n-2);
@@ -139,11 +119,7 @@ class FibTask: public tbb::task {
 
             CilkShutdown();
 
-#if _WIN32 || _WIN64
-            FreeLibrary(hLib);
-#else
-            dlclose(hLib);
-#endif
+            Harness::CloseLibrary(hLib);
         }
         return NULL;
     }
