@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -29,16 +29,14 @@
 #include "harness_graph.h"
 
 #include "tbb/task_scheduler_init.h"
-#include "tbb/spin_mutex.h"
-
-tbb::spin_mutex global_mutex;
+#include "tbb/spin_rw_mutex.h"
 
 #define N 100
 #define MAX_NODES 4
 
 //! Performs test on function nodes with limited concurrency and buffering
 /** Theses tests check:
-    1) that the number of executing copies never exceed the concurreny limit
+    1) that the number of executing copies never exceed the concurrency limit
     2) that the node never rejects
     3) that no items are lost
     and 4) all of this happens even if there are multiple predecessors and successors
@@ -58,6 +56,11 @@ struct parallel_put_until_limit : private NoAssign {
     }
 
 };
+
+template<typename IO>
+struct pass_through {
+    IO operator()(const IO& i) { return i; }
+};
      
 template< typename InputType, typename OutputType, typename Body >
 void buffered_levels( size_t concurrency, Body body ) {
@@ -68,15 +71,24 @@ void buffered_levels( size_t concurrency, Body body ) {
 
    // Set the execute_counter back to zero in the harness
    harness_graph_executor<InputType, OutputType>::execute_count = 0;
+   // Set the number of current executors to zero.
+   harness_graph_executor<InputType, OutputType>::current_executors = 0;
    // Set the max allowed executors to lc.  There is a check in the functor to make sure this is never exceeded.
    harness_graph_executor<InputType, OutputType>::max_executors = lc;
 
-   // Create the function_node with the appropriate concurreny level, and use default buffering
+   // Create the function_node with the appropriate concurrency level, and use default buffering
    tbb::flow::function_node< InputType, OutputType > exe_node( g, lc, body );
+   tbb::flow::function_node<InputType, InputType> pass_thru( g, tbb::flow::unlimited, pass_through<InputType>());
    
-   //Create a vector of identical exe_nodes
+   // Create a vector of identical exe_nodes and pass_thrus
    std::vector< tbb::flow::function_node< InputType, OutputType > > exe_vec(2, exe_node);
+   std::vector< tbb::flow::function_node< InputType, InputType > > pass_thru_vec(2, pass_thru);
+   // Attach each pass_thru to its corresponding exe_node
+   for (size_t node_idx=0; node_idx<exe_vec.size(); ++node_idx) {
+       tbb::flow::make_edge(pass_thru_vec[node_idx], exe_vec[node_idx]);
+   }
 
+   // TODO: why the test is executed serially for the node pairs, not concurrently?
    for (size_t node_idx=0; node_idx<exe_vec.size(); ++node_idx) {
    // For num_receivers = 1 to MAX_NODES
    for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
@@ -89,11 +101,11 @@ void buffered_levels( size_t concurrency, Body body ) {
         // Do the test with varying numbers of senders
         harness_counting_sender<InputType> *senders = NULL;
         for (size_t num_senders = 1; num_senders <= MAX_NODES; ++num_senders ) {
-            // Create num_senders senders, set there message limit each to N, and connect them to the exe_vec[node_idx]
+            // Create num_senders senders, set there message limit each to N, and connect them to pass_thru_vec[node_idx]
             senders = new harness_counting_sender<InputType>[num_senders];
             for (size_t s = 0; s < num_senders; ++s ) {
                senders[s].my_limit = N;
-               tbb::flow::make_edge( senders[s], exe_vec[node_idx] );
+               tbb::flow::make_edge( senders[s], pass_thru_vec[node_idx] );
             }
 
             // Initialize the receivers so they know how many senders and messages to check for
@@ -105,11 +117,11 @@ void buffered_levels( size_t concurrency, Body body ) {
             NativeParallelFor( (int)num_senders, parallel_put_until_limit<InputType>(senders) );
             g.wait_for_all();
 
-            // cofirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times 
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
                 ASSERT( n == N, NULL ); 
-                ASSERT( senders[s].my_receiver == &exe_vec[node_idx], NULL );
+                ASSERT( senders[s].my_receiver == &pass_thru_vec[node_idx], NULL );
             }
             // validate the receivers
             for (size_t r = 0; r < num_receivers; ++r ) {
@@ -127,9 +139,9 @@ void buffered_levels( size_t concurrency, Body body ) {
             receivers[r].validate();
         }
         delete [] receivers;
-    }
-    } 
-    }
+    } // for num_receivers
+    } // for node_idx
+    } // for concurrency level lc
 }
 
 const size_t Offset = 123;
@@ -215,12 +227,11 @@ void buffered_levels_with_copy( size_t concurrency ) {
 
 template< typename InputType, typename OutputType >
 void run_buffered_levels( int c ) {
-    harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::max_executors = c;
     #if __TBB_LAMBDAS_PRESENT
-    buffered_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func(i); } );
+    buffered_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::func(i); } );
     #endif
-    buffered_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func );
-    buffered_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::functor() );
+    buffered_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType>::func );
+    buffered_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType>::functor() );
     buffered_levels_with_copy<InputType,OutputType>( c );
 }
 
@@ -239,7 +250,13 @@ void concurrency_levels( size_t concurrency, Body body ) {
 
    for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
    tbb::flow::graph g;
-   harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::execute_count = 0;
+
+   // Set the execute_counter back to zero in the harness
+   harness_graph_executor<InputType, OutputType>::execute_count = 0;
+   // Set the number of current executors to zero.
+   harness_graph_executor<InputType, OutputType>::current_executors = 0;
+   // Set the max allowed executors to lc. There is a check in the functor to make sure this is never exceeded.
+   harness_graph_executor<InputType, OutputType>::max_executors = lc;
 
    tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > exe_node( g, lc, body );
 
@@ -254,9 +271,10 @@ void concurrency_levels( size_t concurrency, Body body ) {
         harness_counting_sender<InputType> *senders = NULL;
 
         for (size_t num_senders = 1; num_senders <= MAX_NODES; ++num_senders ) {
+            senders = new harness_counting_sender<InputType>[num_senders];
             {
-                // lock m to prevent exe_node from finishing
-                tbb::spin_mutex::scoped_lock l( harness_graph_executor< InputType, OutputType, tbb::spin_mutex >::mutex );
+                // Exclusively lock m to prevent exe_node from finishing
+                tbb::spin_rw_mutex::scoped_lock l( harness_graph_executor<InputType, OutputType>::template mutex_holder<tbb::spin_rw_mutex>::mutex );
 
                 // put to lc level, it will accept and then block at m
                 for ( size_t c = 0 ; c < lc ; ++c ) {
@@ -265,7 +283,6 @@ void concurrency_levels( size_t concurrency, Body body ) {
                 // it only accepts to lc level
                 ASSERT( exe_node.try_put( InputType() ) == false, NULL );
 
-                senders = new harness_counting_sender<InputType>[num_senders];
                 for (size_t s = 0; s < num_senders; ++s ) {
                    // register a sender
                    senders[s].my_limit = N;
@@ -276,13 +293,13 @@ void concurrency_levels( size_t concurrency, Body body ) {
             // wait for graph to settle down
             g.wait_for_all();
 
-            // cofirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times 
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
                 ASSERT( n == N, NULL ); 
                 ASSERT( senders[s].my_receiver == &exe_node, NULL );
             }
-            // cofirm that each receivers got N * num_senders + the initial lc puts
+            // confirm that each receivers got N * num_senders + the initial lc puts
             for (size_t r = 0; r < num_receivers; ++r ) {
                 size_t n = receivers[r].my_count;
                 ASSERT( n == num_senders*N+lc, NULL );
@@ -306,12 +323,11 @@ void concurrency_levels( size_t concurrency, Body body ) {
 
 template< typename InputType, typename OutputType >
 void run_concurrency_levels( int c ) {
-    harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::max_executors = c;
     #if __TBB_LAMBDAS_PRESENT
-    concurrency_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func(i); } );
+    concurrency_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::template tfunc<tbb::spin_rw_mutex>(i); } );
     #endif
-    concurrency_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func );
-    concurrency_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::functor() );
+    concurrency_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType>::template tfunc<tbb::spin_rw_mutex> );
+    concurrency_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType>::template tfunctor<tbb::spin_rw_mutex>() );
 }
 
 
@@ -396,7 +412,7 @@ void test_function_node_with_continue_msg_as_input() {
     // If this function terminates, then this test is successful
     tbb::flow::graph g;
 
-    tbb::flow::broadcast_node<tbb::flow::continue_msg> Start;
+    tbb::flow::broadcast_node<tbb::flow::continue_msg> Start(g);
 
     tbb::flow::function_node<tbb::flow::continue_msg, int, tbb::flow::rejecting> FN1( g, tbb::flow::serial, continue_msg_to_int(42));
     tbb::flow::function_node<tbb::flow::continue_msg, int, tbb::flow::rejecting> FN2( g, tbb::flow::serial, continue_msg_to_int(43));
@@ -424,7 +440,6 @@ void test_concurrency(int num_threads) {
 }
 
 int TestMain() { 
-    current_executors = 0;
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
