@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -40,10 +40,12 @@ struct serial_receiver : public tbb::flow::receiver<T> {
 
    serial_receiver() : next_value(T(0)) {}
 
-   /* override */ bool try_put( const T &v ) {
+   /* override */ tbb::task *try_put_task( const T &v ) {
        ASSERT( next_value++  == v, NULL );
-       return true;
+       return const_cast<tbb::task *>(tbb::flow::interface6::SUCCESSFULLY_ENQUEUED);
    }
+
+   /*override*/void reset_receiver() {next_value = T(0);}
 };
 
 template< typename T >
@@ -53,10 +55,12 @@ struct parallel_receiver : public tbb::flow::receiver<T> {
 
    parallel_receiver() { my_count = 0; }
 
-   /* override */ bool try_put( const T & ) {
+   /* override */ tbb::task *try_put_task( const T &/*v*/ ) {
        ++my_count;
-       return true;
+       return const_cast<tbb::task *>(tbb::flow::interface6::SUCCESSFULLY_ENQUEUED);
    }
+
+   /*override*/void reset_receiver() {my_count = 0;}
 };
 
 template< typename T >
