@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -34,6 +34,8 @@
 // iff __STDC_LIMIT_MACROS pre-defined
 #define __STDC_LIMIT_MACROS 1
 
+#define HARNESS_TBBMALLOC_THREAD_SHUTDOWN 1
+
 #include "harness.h"
 #include "harness_barrier.h"
 
@@ -42,7 +44,7 @@
 #undef DO_ITT_NOTIFY
 #endif
 
-#define __TBB_MALLOC_WHITEBOX_TEST 1 // to get access to LOC internals
+#define __TBB_MALLOC_WHITEBOX_TEST 1 // to get access to allocator internals
 // help trigger rare race condition
 #define WhiteboxTestingYield() (__TBB_Yield(), __TBB_Yield(), __TBB_Yield(), __TBB_Yield())
 
@@ -216,20 +218,24 @@ public:
     }
 };
 
-class FreeBlockPoolHit: NoAssign {
-    // to trigger possible leak for both cleanup on pool overflow 
-    // and on thread termination
-    static const int ITERS = 2*FreeBlockPool::POOL_HIGH_MARK;
+class LocalCachesHit: NoAssign {
+    // set ITERS to trigger possible leak of backreferences
+    // during cleanup on cache overflow and on thread termination
+    static const int ITERS = 2*(FreeBlockPool::POOL_HIGH_MARK +
+                                LocalLOC::LOC_HIGH_MARK);
 public:
-    FreeBlockPoolHit() {}
+    LocalCachesHit() {}
     void operator()(int) const {
-        void *objs[ITERS];
-
-        for (int i=0; i<ITERS; i++)
-            objs[i] = scalable_malloc(minLargeObjectSize-1);
-        for (int i=0; i<ITERS; i++)
-            scalable_free(objs[i]);
+        void *objsSmall[ITERS], *objsLarge[ITERS];
 
+        for (int i=0; i<ITERS; i++) {
+            objsSmall[i] = scalable_malloc(minLargeObjectSize-1);
+            objsLarge[i] = scalable_malloc(minLargeObjectSize);
+        }
+        for (int i=0; i<ITERS; i++) {
+            scalable_free(objsSmall[i]);
+            scalable_free(objsLarge[i]);
+        }
 #ifdef USE_WINTHREAD
         // Under Windows DllMain is used for mallocThreadShutdownNotification
         // calling. As DllMain is not used during whitebox testing,
@@ -247,11 +253,6 @@ static size_t allocatedBackRefCount()
     return cnt;
 }
 
-static void cleanObjectCache()
-{
-    defaultMemPool->extMemPool.hardCachesCleanup();
-}
-
 class TestInvalidBackrefs: public SimpleBarrier {
 #if __ANDROID__
     // Android requires lower iters due to lack of virtual memory.
@@ -313,14 +314,16 @@ void TestBackRef() {
     int sustLastUsed = backRefMaster->lastUsed;
     NativeParallelFor( 1, BackRefWork() );
     ASSERT(sustLastUsed == backRefMaster->lastUsed, "backreference leak detected");
-    
-    // check leak of back references while per-thread small object pool is in use
-    // warm up need to cover bootStrapMalloc call
-    NativeParallelFor( 1, FreeBlockPoolHit() );
+
+    // check leak of back references while per-thread caches are in use
+    // warm up needed to cover bootStrapMalloc call
+    NativeParallelFor( 1, LocalCachesHit() );
     beforeNumBackRef = allocatedBackRefCount();
-    NativeParallelFor( 1, FreeBlockPoolHit() );
+    NativeParallelFor( 2, LocalCachesHit() );
+    int res = scalable_allocation_command(TBBMALLOC_CLEAN_ALL_BUFFERS, NULL);
+    ASSERT(res == TBBMALLOC_OK, NULL);
     afterNumBackRef = allocatedBackRefCount();
-    ASSERT(beforeNumBackRef==afterNumBackRef, "backreference leak detected");
+    ASSERT(beforeNumBackRef>=afterNumBackRef, "backreference leak detected");
 
     // This is a regression test against race condition between backreference
     // extension and checking invalid BackRefIdx.
@@ -377,6 +380,21 @@ int putMallocMem(intptr_t /*pool_id*/, void *ptr, size_t bytes)
     return 0;
 }
 
+class StressLOCacheWork: NoAssign {
+    rml::MemoryPool *mallocPool;
+public:
+    StressLOCacheWork(rml::MemoryPool *mallocPool) : mallocPool(mallocPool) {}
+    void operator()(int) const {
+        for (size_t sz=minLargeObjectSize; sz<1*1024*1024;
+             sz+=LargeObjectCache::largeBlockCacheStep) {
+            void *ptr = pool_malloc(mallocPool, sz);
+            ASSERT(ptr, "Memory was not allocated");
+            memset(ptr, sz, sz);
+            pool_free(mallocPool, ptr);
+        }
+    }
+};
+
 void TestPools() {
     rml::MemPoolPolicy pol(getMem, putMem);
     size_t beforeNumBackRef, afterNumBackRef;
@@ -388,7 +406,7 @@ void TestPools() {
     pool_destroy(pool1);
     pool_destroy(pool2);
 
-    cleanObjectCache();
+    scalable_allocation_command(TBBMALLOC_CLEAN_ALL_BUFFERS, NULL);
     beforeNumBackRef = allocatedBackRefCount();
     rml::MemoryPool *fixedPool;
 
@@ -440,16 +458,12 @@ void TestPools() {
     pool_free(fixedPool, largeObj);
 
     // provoke large object cache cleanup and hope no leaks occurs
-    for (size_t sz=minLargeObjectSize; sz<1*1024*1024; sz+=LargeObjectCache::largeBlockCacheStep) {
-        ptr = pool_malloc(mallocPool, sz);
-        ASSERT(ptr, "Memory was not allocated");
-        memset(ptr, sz, sz);
-        pool_free(mallocPool, ptr);
-    }
+    for( int p=MaxThread; p>=MinThread; --p )
+        NativeParallelFor( p, StressLOCacheWork(mallocPool) );
     pool_destroy(mallocPool);
     pool_destroy(fixedPool);
 
-    cleanObjectCache();
+    scalable_allocation_command(TBBMALLOC_CLEAN_ALL_BUFFERS, NULL);
     afterNumBackRef = allocatedBackRefCount();
     ASSERT(beforeNumBackRef==afterNumBackRef, "backreference leak detected");
 
@@ -468,7 +482,7 @@ void TestPools() {
         ASSERT(loc->getUsedSize(), NULL);
         pool_free(mallocPool, p[3]);
         ASSERT(loc->getLOCSize() < 3*(minLargeObjectSize+LargeObjectCache::largeBlockCacheStep), NULL);
-        const size_t maxLocalLOCSize = LocalLOC<3,30>::getMaxSize();
+        const size_t maxLocalLOCSize = LocalLOCImpl<3,30>::getMaxSize();
         ASSERT(loc->getUsedSize() <= maxLocalLOCSize, NULL);
         for (int i=0; i<3; i++)
             p[i] = pool_malloc(mallocPool, minLargeObjectSize+i*LargeObjectCache::largeBlockCacheStep);
@@ -485,7 +499,7 @@ void TestPools() {
     // To test LOC we need bigger lists than released by current LocalLOC
     //   in production code. Create special LocalLOC.
     {
-        LocalLOC<2, 20> lLOC;
+        LocalLOCImpl<2, 20> lLOC;
         pool_create_v1(0, &pol, &mallocPool);
         rml::internal::ExtMemoryPool *mPool = &((rml::internal::MemoryPool*)mallocPool)->extMemPool;
         const LargeObjectCache *loc = &((rml::internal::MemoryPool*)mallocPool)->extMemPool.loc;
@@ -498,7 +512,7 @@ void TestPools() {
             ret = lLOC.put(((LargeObjectHdr*)o - 1)->memoryBlock, mPool);
             ASSERT(ret, NULL);
         }
-        lLOC.clean(mPool);
+        lLOC.externalCleanup(mPool);
         ASSERT(!loc->getUsedSize(), NULL);
 
         pool_destroy(mallocPool);
@@ -651,12 +665,139 @@ void TestBitMask()
     ASSERT(mask.getMinTrue(201) == -1, NULL);
 }
 
+size_t getMemSize()
+{
+    return defaultMemPool->extMemPool.backend.getTotalMemSize();
+}
+
+class CheckNotCached {
+    size_t memSize;
+public:
+    CheckNotCached(size_t memSize) : memSize(memSize) {}
+    void operator() () const {
+        int res = scalable_allocation_mode(TBBMALLOC_SET_SOFT_HEAP_LIMIT, 1);
+        ASSERT(res == TBBMALLOC_OK, NULL);
+        ASSERT(getMemSize() == memSize, NULL);
+    }
+};
+
+class RunTestHeapLimit: public SimpleBarrier {
+    size_t memSize;
+public:
+    RunTestHeapLimit(size_t memSize) : memSize(memSize) {}
+
+    void operator()( int /*mynum*/ ) const {
+        CheckNotCached checkNotCached(memSize);
+
+        for (size_t n = minLargeObjectSize; n < 5*1024*1024; n += 128*1024)
+            scalable_free(scalable_malloc(n));
+        barrier.wait(checkNotCached);
+    }
+};
+
+void TestHeapLimit()
+{
+    if(!isMallocInitialized()) doInitialization();
+    // tiny limit to stop caching
+    int res = scalable_allocation_mode(TBBMALLOC_SET_SOFT_HEAP_LIMIT, 1);
+    ASSERT(res == TBBMALLOC_OK, NULL);
+     // provoke bootstrap heap initialization before recording memory size
+    scalable_free(scalable_malloc(8));
+    size_t n, sizeBefore = getMemSize();
+
+    // Try to provoke call to OS for memory to check that
+    // requests are not fulfilled from caches.
+    // Single call is not enough here because of backend fragmentation.
+    for (n = minLargeObjectSize; n < 10*1024*1024; n += 16*1024) {
+        void *p = scalable_malloc(n);
+        bool leave = (sizeBefore != getMemSize());
+        scalable_free(p);
+        if (leave)
+            break;
+        ASSERT(sizeBefore == getMemSize(), "No caching expected");
+    }
+    ASSERT(n < 10*1024*1024, "scalable_malloc doesn't provoke OS request for memory, "
+           "is some internal cache still used?");
+    // estimate number of objects in single bootstrap block
+    int objInBootstrapHeapBlock = (slabSize-2*estimatedCacheLineSize)/sizeof(TLSData);
+    // When we have more threads than objects in bootstrap heap block,
+    // additional block can be allocated from a region that is different
+    // from the original region. Thus even after all caches cleaned,
+    // we unable to reach sizeBefore.
+    ASSERT_WARNING(MaxThread<=objInBootstrapHeapBlock,
+        "The test might fail for larger thread number, "
+        "as bootstrap heap is not released till size checking.");
+    for( int p=MaxThread; p>=MinThread; --p ) {
+        RunTestHeapLimit::initBarrier( p );
+        NativeParallelFor( p, RunTestHeapLimit(sizeBefore) );
+    }
+    // it's try to match limit as well as set limit, so call here
+    res = scalable_allocation_mode(TBBMALLOC_SET_SOFT_HEAP_LIMIT, 1);
+    ASSERT(res == TBBMALLOC_OK, NULL);
+    size_t m = getMemSize();
+    ASSERT(sizeBefore == m, NULL);
+    // restore default
+    res = scalable_allocation_mode(TBBMALLOC_SET_SOFT_HEAP_LIMIT, 0);
+    ASSERT(res == TBBMALLOC_OK, NULL);
+}
+
 void checkNoHugePages()
 {
     ASSERT(!hugePages.enabled, "scalable_allocation_mode "
            "must have priority over environment variable");
 }
 
+/*---------------------------------------------------------------------------*/
+// The regression test against a bug in TBBMALLOC_CLEAN_ALL_BUFFERS allocation
+// command. When cleanup is requested the backend should process the queue of
+// postponed coalescing requests otherwise not all unsued memory might be
+// deallocated.
+
+const size_t alloc_size = 16*1024;
+const int total_alloc_size = 100 * 1024 * 1024;
+const int num_allocs = total_alloc_size / alloc_size;
+void *ptrs[num_allocs];
+
+tbb::atomic<int> deallocs_counter;
+
+struct TestCleanAllBuffersDeallocate : public SimpleBarrier {
+    void operator() ( int ) const {
+        barrier.wait();
+        for( int i = deallocs_counter++; i < num_allocs; i = deallocs_counter++ )
+           scalable_free( ptrs[i] );
+    }
+};
+
+// The idea is to allocate a set of objects and then deallocate them in random
+// order in parallel to force occuring conflicts in backend during coalescing.
+// Thus if the backend does not check the queue of postponed coalescing
+// requests it will not be able to unmap all memory and a memory leak will be
+// observed.
+void TestCleanAllBuffers() {
+    const int num_threads = 8;
+    // Clean up if something was allocated before the test
+    scalable_allocation_command(TBBMALLOC_CLEAN_ALL_BUFFERS,0);
+
+    size_t memory_in_use_before = getMemSize();
+    for ( int i=0; i<num_allocs; ++i ) {
+        ptrs[i] = scalable_malloc( alloc_size );
+        ASSERT( ptrs[i] != NULL, "scalable_malloc has return zero." );
+    }
+    deallocs_counter = 0;
+    TestCleanAllBuffersDeallocate::initBarrier(num_threads);
+    NativeParallelFor(num_threads, TestCleanAllBuffersDeallocate());
+    if ( defaultMemPool->extMemPool.backend.coalescQ.blocksToFree == NULL )
+        REPORT( "Warning: The queue of postponed coalescing requests is empty. Unable to create the condition for bug reproduction.\n" );
+    ASSERT( scalable_allocation_command(TBBMALLOC_CLEAN_ALL_BUFFERS,0) == TBBMALLOC_OK, "The cleanup request has not cleaned anithing." );
+    size_t memory_in_use_after = getMemSize();
+
+    REMARK( "memory_in_use_before = %ld\nmemory_in_use_after = %ld\n", memory_in_use_before, memory_in_use_after );
+
+    size_t memory_leak = memory_in_use_after - memory_in_use_before;
+    ASSERT( memory_leak == 0, "The backend has not processed the queue of postponed coalescing requests during cleanup." );
+}
+/*---------------------------------------------------------------------------*/
+
 int TestMain () {
     scalable_allocation_mode(USE_HUGE_PAGES, 0);
 #if !_XBOX && !__TBB_WIN8UI_SUPPORT
@@ -682,5 +823,7 @@ int TestMain () {
     TestLargeObjectCache();
     TestObjectRecognition();
     TestBitMask();
+    TestHeapLimit();
+    TestCleanAllBuffers();
     return Harness::Done;
 }
