@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -335,7 +335,7 @@ int test_serial() {
 
 int TestMain() { 
     tbb::tick_count start = tbb::tick_count::now(), stop;
-    for (int p = 1; p < 4; ++p) {
+    for (int p = 2; p <= 4; ++p) {
         tbb::task_scheduler_init init(p);
         test_serial<int>();
         test_reservation<int>(p);
