@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -38,20 +38,7 @@
     switching producer thread, and the check is repeated.
 */
 
-#if HARNESS_USE_PROXY
-
-// The test includes injects scheduler directly, so skip it when proxy tested.
-
-#undef HARNESS_USE_PROXY
-#include "harness.h"
-#undef __TBB_DYNAMIC_LOAD_ENABLED
-#include "harness_tbb_independence.h"
-
-int TestMain () {
-    return Harness::Skipped;
-}
-
-#else // HARNESS_USE_PROXY
+#define HARNESS_DEFAULT_MIN_THREADS -1
 
 #define  __TBB_COUNT_TASK_NODES 1
 #include "harness_inject_scheduler.h"
@@ -80,7 +67,6 @@ tbb::internal::scheduler* Producer;
 
 #include "tbb/task_scheduler_init.h"
 
-#define HARNESS_DEFAULT_MIN_THREADS -1
 #include "harness.h"
 
 using namespace tbb;
@@ -100,7 +86,7 @@ class TaskGenerator: public tbb::task {
     const int my_child_count;
     int my_depth;
 public:
-    TaskGenerator(int child_count, int depth) : my_child_count(child_count), my_depth(depth) {
+    TaskGenerator(int child_count, int d) : my_child_count(child_count), my_depth(d) {
         ASSERT(my_child_count>1, "The TaskGenerator should produce at least two children");
     }
     /*override*/ tbb::task* execute() {
@@ -290,6 +276,3 @@ int TestMain () {
     TestTaskReclamation();
     return Harness::Done;
 }
-
-#endif  // HARNESS_USE_PROXY
-
